==> xmms-crossfade-0.2.4.tar.gz: xmms-crossfade-0.2.4/effect.c <==
/* 
 *  XMMS Crossfade Plugin
 *  Copyright (C) 2000-2001  Peter Eisenlohr <p.eisenlohr@gmx.net>
 *
 *  based on the original OSS Output Plugin
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "effect.h"


void
effect_init(effect_context_t *ec,
	    EffectPlugin     *ep)
{
  memset(ec, 0, sizeof(*ec));
  ec->ep = ep;
}

void
effect_set_plugin(effect_context_t *ec,
		  EffectPlugin     *ep)
{
  ec->ep = ep;
}

gint
effect_flow(effect_context_t *ec,
	    gpointer         *buffer,
	    gint              length,
	    format_t         *format,
	    gboolean          allow_format_change)
{
  EffectPlugin *ep;

  if(effects_enabled()) {
    ep = ec->ep ? ec->ep : get_current_effect_plugin();
    if(ep != ec->last_ep) {
      DEBUG(("[crossfade] effect: plugin: %s\n",
	     ep ? (ep->description ? ep->description : "<unnamed>") 
	     : "<none>"));
      ec->last_ep = ep;
    }
    if(ep == NULL) return length;

    if(ep->query_format) {
      AFormat fmt  = format->fmt;
      gint    rate = format->rate;
      gint    nch  = format->nch;
      
      ep->query_format(&fmt, &rate, &nch);
      if(!ec->is_active
	 || (ec->fmt  != fmt)
	 || (ec->rate != rate)
	 || (ec->nch  != nch)) {
	if(!allow_format_change && ((fmt  != format->fmt) ||
				    (rate != format->rate) ||
				    (nch  != format->nch))) {
	  DEBUG(("[crossfade] effect: format mismatch "
		 "(fmt=%d rate=%d nch=%d)!\n", fmt, rate, nch));
	  ec->is_valid = FALSE;
	}
	else {
	  if(setup_format(fmt, rate, nch, &ec->format) < 0) {
	    DEBUG(("[crossfade] effect: format not supported "
		   "(fmt=%d rate=%d nch=%d)!\n", fmt, rate, nch));
	    ec->is_valid = FALSE;
	  }
	  else {
	    DEBUG(("[crossfade] effect: plugin enabled "
		   "(fmt=%d rate=%d nch=%d)\n", fmt, rate, nch));
	    ec->is_valid = TRUE;
	  }
	}

	ec->is_active = TRUE;
	ec->fmt       = fmt;
	ec->rate      = rate;
	ec->nch       = nch;
      }
      if(ec->is_valid && ep->mod_samples) {
	length = ep->mod_samples(buffer, length, format->fmt, format->rate, format->nch);
	if(allow_format_change) *format = ec->format;
      }
    }
    else {
      if(!ec->is_active) {
	DEBUG(("[crossfade] effect: plugin enabled (default format)\n"));
	ec->is_active = TRUE;
      }
      if(ep->mod_samples)
	length = ep->mod_samples(buffer, length, format->fmt, format->rate, format->nch);
    }
  }
  else if(ec->is_active) {
    DEBUG(("[crossfade] effect: plugin disabled\n"));
    ec->is_active = FALSE;
  }

  return length;
}

void
effect_free(effect_context_t *ec)
{
  /* nothing to do */
}

==> xmms-crossfade_0.2.9.orig.tar.gz: xmms-crossfade-0.2.9/effect.c <==
/* 
 *  XMMS Crossfade Plugin
 *  Copyright (C) 2000-2001  Peter Eisenlohr <p.eisenlohr@gmx.net>
 *
 *  based on the original OSS Output Plugin
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "effect.h"


void
effect_init(effect_context_t *ec,
	    EffectPlugin     *ep)
{
  memset(ec, 0, sizeof(*ec));
  effect_set_plugin(ec, ep);
}

void
effect_set_plugin(effect_context_t *ec,
		  EffectPlugin     *ep)
{
  ec->use_xmms_plugin = (ep == EFFECT_USE_XMMS_PLUGIN);
  ec->ep = ec->use_xmms_plugin ? NULL : ep;
}

gint
effect_flow(effect_context_t *ec,
	    gpointer         *buffer,
	    gint              length,
	    format_t         *format,
	    gboolean          allow_format_change)
{
  EffectPlugin *ep;

  if(ec->use_xmms_plugin ? effects_enabled() : (ec->ep != NULL)) {
    ep = ec->use_xmms_plugin ? get_current_effect_plugin() : ec->ep;
    if(ep != ec->last_ep) {
      DEBUG(("[crossfade] effect: plugin: \"%s\"%s\n",
	     ep ? (ep->description ? ep->description : "<unnamed>") : "<none>",
	     ec->use_xmms_plugin ? " (XMMS)" : ""));
      ec->last_ep = ep;
    }
    if(ep == NULL) return length;

    if(ep->query_format) {
      AFormat fmt  = format->fmt;
      gint    rate = format->rate;
      gint    nch  = format->nch;
      
      ep->query_format(&fmt, &rate, &nch);
      if(!ec->is_active
	 || (ec->fmt  != fmt)
	 || (ec->rate != rate)
	 || (ec->nch  != nch)) {
	if(!allow_format_change && ((fmt  != format->fmt) ||
				    (rate != format->rate) ||
				    (nch  != format->nch))) {
	  DEBUG(("[crossfade] effect: format mismatch "
		 "(fmt=%d rate=%d nch=%d)!\n", fmt, rate, nch));
	  ec->is_valid = FALSE;
	}
	else {
	  if(setup_format(fmt, rate, nch, &ec->format) < 0) {
	    DEBUG(("[crossfade] effect: format not supported "
		   "(fmt=%d rate=%d nch=%d)!\n", fmt, rate, nch));
	    ec->is_valid = FALSE;
	  }
	  else {
	    DEBUG(("[crossfade] effect: plugin enabled "
		   "(fmt=%d rate=%d nch=%d)\n", fmt, rate, nch));
	    ec->is_valid = TRUE;
	  }
	}

	ec->is_active = TRUE;
	ec->fmt       = fmt;
	ec->rate      = rate;
	ec->nch       = nch;
      }
      if(ec->is_valid && ep->mod_samples) {
	length = ep->mod_samples(buffer, length, format->fmt, format->rate, format->nch);
	if(allow_format_change) format_copy(format, &ec->format);
      }
    }
    else {
      if(!ec->is_active) {
	DEBUG(("[crossfade] effect: plugin enabled (default format)\n"));
	ec->is_active = TRUE;
      }
      if(ep->mod_samples)
	length = ep->mod_samples(buffer, length, format->fmt, format->rate, format->nch);
    }
  }
  else if(ec->is_active) {
    DEBUG(("[crossfade] effect: plugin disabled\n"));
    ec->is_active = FALSE;
  }

  return length;
}

void
effect_free(effect_context_t *ec)
{
  /* nothing to do */
}
